using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Random;

namespace EvoAlg.Ue3
{
        //Populationstyp ist die Klasse TSPPopulation
        //IndividuumsTyp ist die Klasse IntVectorMinimize
        
        //Die Attribute in "[]" müssen vor der Klassendeklaration stehen.
        
        //Der Name mit dem Instanzen dieses Moduls standardmäßig identifiziert werden:
        [Identifier("Beispielmodul")]
        
        [Author("Uebungsteilnehmer(in)")]
        [Description("Dieses Modul dient als selbsterklärendes Beispiel")]
        
        //Schlüssel- Defaultwertpaare für einen von diesem Modul benötigten Parameter
        //Weitere (unwichtigere) ParameterTypen befinden sich unter CoreApi/Parameters
        [IntParameter("BeispielIntParameter", 23)]
        [DoubleParameter("BeispielFloatParameter", 3.1415928d)]

        //Die für Übungszwecke geschriebenen Module sollten alle von einer der Klassen in
        //EvoLab.CoreApi.ModuleTemplates abgeleitet sein.
        //Bei diesen Klassen muss jeweils Individuumstyp und Populationstyp als generischer Parameter mitgegeben werden.
        public class BeispielMutation : EvoLab.CoreApi.ModuleTemplates.MutationBase<PermutationPopulation> {
                
                //Wird einmal vor Beginn des Zyklus aufgerufen und kann entsprechend für
                //einmalige Initialisierungen verwendet werden.
                // name - der tatsächlich zugewiesene Identifier für diese Instanz
                // population - enthält u.a. die Individuen
                // world - enthält die Parameter und Zugriff auf diverse -mehr oder weniger benötigte- Funktionen
                public override void Init (string name, PermutationPopulation population, IWorld world)
                {
                        //Die Basismethode speichert lediglich name, population und world in 3 gleichnamigen Feldern ab
                        //Bei spezialisierteren Templates trifft sie ggf. noch weitere Vorbereitungen.
                        base.Init (name, population, world);
                        
                        //Irgendwelche Vorberechnungen..
                        
                        //Einlesen der Parameter:
                        beispielInt = world.GetLocalInteger(name, "BeispielIntParameter");
                        beispielFloat = world.GetLocalDouble(name, "BeispielFloatParameter");
                }
                
                int beispielInt;
                double beispielFloat;

                //Wird einmal pro Generation an der festgelegten Stelle des Zyklus aufgerufen
                public override void Run ()
                {
                        //Die Referenz auf das Array population.Pop sollte generell zwischen Zyklen
                        //nicht zwischengespeichert werden.
                        //Mutiert jedes Individuum
                        foreach (IntVector ind in population.Pop) {
                                //Bringt die Permutation des Individuums durcheinander (tm) ;-)
                                //Für Zufallszahlen sollte immer world.RandomNumberGenerator verwendet werden.
                                EvoLab.CoreApi.Operators.Shuffler.Shuffle<int>(world.RandomNumberGenerator, ind.vector);
                        }
                }

        }
}
