/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;

namespace EvoLab.Common {
    
    public sealed class FileHelper {
        private FileHelper () {}
        
        public static bool FilesAreEqual(string file1, string file2) {
            using (FileStream fs1 = File.OpenRead(file1)) {
                using (FileStream fs2 = File.OpenRead(file2)) {
                    if (fs1.Length != fs2.Length) {
                        return false;
                    }
                    while (fs1.Position < fs1.Length) {
                        if (fs1.ReadByte() != fs2.ReadByte()) {
                            return false;
                        }
                    }
                    return true;
                }
            }
        }
        
        public static void CopyFileIfNotExists (string sourceFile, string targetFile)
        {
            if (!File.Exists(targetFile)) {
                File.Copy(sourceFile, targetFile);
            }
        }
        
        public static void CreateDirectoryIfNotExists (string directory)
        {
            if (!Directory.Exists(directory)) {
                Directory.CreateDirectory(directory);
            }
        }
    }
}
