/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

/// <summary>
/// Dieser Namespace enthält structs zur Beschreibung der ModulParameter.
///
/// Das Interface IParameterInfo fasst all diese structs zu einem gemeinsamen
/// Typ zusammen.
///
/// Es sollte hier zu jedem Attribut in EvoLab.Api.CoreApi
/// ein entsprechendes struct geben.
/// </summary>
/// @cond
/// <author>Daniel Hofmann</author>
/// <version>$Revision: 2625 $</version>
/// <id>$Id: ParameterInfo.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
/// @endcond
namespace EvoLab.Common.ModuleInfo
{

    public interface IParameterInfo
    {
    }

    [Serializable]
    public struct AuthorInfo : IParameterInfo
    {
        public string author;
        public AuthorInfo(string author)
        {
            this.author = author;
        }
    }

    [Serializable]
    public struct DescriptionInfo : IParameterInfo
    {
        public string description;
        public DescriptionInfo(string description)
        {
            this.description = description;
        }
    }

    public abstract class AbstractParameterInfo<T> : IParameterInfo {
        public string name;
        public T value;
        
        public AbstractParameterInfo(string name, T value)
        {
            this.name = name;
            this.value = value;
        }
    }

    public class BooleanInfo : AbstractParameterInfo<bool>
    {
        public BooleanInfo(string name, bool value) : base (name, value)
        {
        }
    }

    public class DoubleInfo : AbstractParameterInfo<double>
    {
        public DoubleInfo(string name, double value) : base(name, value)
        {
        }
    }

    public class FloatInfo : AbstractParameterInfo<float>
    {
        public FloatInfo(string name, float value) : base (name, value)
        {
        }
    }

    public class IntInfo : AbstractParameterInfo<int>
    {
        public IntInfo(string name, int value) : base (name, value)
        {
        }
    }

    public class IntSequenceInfo : AbstractParameterInfo<int[]>
    {
        public IntSequenceInfo(string name, int[] value) : base (name, value)
        {
        }
    }

    public class StringInfo : AbstractParameterInfo<string>
    {
        public StringInfo(string name, string value) : base (name, value)
        {
        }
    }

    public class OperatorInfo : AbstractParameterInfo<string[]>
    {
        public OperatorInfo(string name, string[] value) : base (name, value)
        {
        }
    }

}
