/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace EvoLab.Common
{
    /// @cond
    /// <author>Vasilios Grigas</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: OutputDataSet.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    /// <summary>
    /// Diese Klasse wird dazu verwendet die Daten, die ein Output-Modul auf die
    /// Festplatte ausgegeben ber die Kommunikations-Schnittstelle zu senden.
    /// Ein Objekt dieser Klasse entspricht dabei einem Datensatz mit Meta-Informationen
    /// wie Erstellungsdatum oder Simulations-ID.
    /// </summary> 
    [Serializable]
    public class OutputDataSet : OutputDataSetInfo, IComparable<OutputDataSet>
    {
        public String myContent;


        /// <summary>
        /// Konstruktor. ber diesen werden alle ab nun schreibgeschtzten Inhalte
        /// des Datensatzes festgelegt. Alle Inhalte bis auf "content" werden
        /// aus der Vaterklasse verwendet.
        /// </summary>
        /// <param name=".SimID">Die ID der Simulation, in welcher dieser Datensatz erstellt wurde.</param>
        /// <param name="date">Das Datum, zu dem dieser Datensatz auf die Festplatte geschrieben wurde.</param>
        /// <param name="number">Die Identifizierungsnummer dieses Datensatzes, welches den Datensatz Simulations-
        /// intern identifiziert.</param>
        /// <param name="content">Die eigentliche Ausgabe des Output-Moduls.</param>
        /// <param name="outputModulIdentifier">Identifiziert das Outputmodul, mit welchem dieser Datensatz geschrieben wurde.</param>
        /// <param name="generationStart">Die erste Generation, deren Output sich in diesem Datensatz befindet.</param>
        /// <param name="generationEnd">Die letzte Generation, deren Output sich in diesem Datensatz befindet. Muss >= generationStart sein.</param>    
        public OutputDataSet(String simID, DateTime createDate, DateTime changeDate, int number, String content,
                             String outputModulIdentifier )
            : base(simID, createDate, changeDate, number, outputModulIdentifier)
        {
            myContent = content;
        }


        public OutputDataSet() { }


        /// <summary>
        /// Die eigentliche Ausgabe des Output-Moduls.
        /// </summary>
        public String content
        {
            get { return myContent; }
        }


        /// <summary>
        /// Die Vergleichsmethode aus dem Interface IComparable(Generisch) um eine
        /// Sortierung von Objekten dieser Klasse nach Modulidentifier und dann Nummer zu ermglichen.
        /// </summary>
        public int CompareTo(OutputDataSet o)
        {
            return base.CompareTo(o);        
        }

    }
}
