/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using EvoLab.Common;


namespace EvoLab.Common.Xml
{
    /// <summary>
    /// </summary> 
    /// @cond
    /// <author>Ting Wang</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: XMLWriter.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    class XMLWriter
    {

        public XMLWriter()
        {
        }
        public static void Save(SimulationInfo si, ModuleInfo[] mis, string fileName)
        {
            string filename = fileName;
            
            //TODO: throw exception if file already exists.
            
            FileStream fs = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.Write);
            

            StreamWriter sw = new StreamWriter(fs);
            XmlTextWriter xw = new XmlTextWriter(sw);
            
            xw.Formatting = Formatting.Indented;
            xw.Indentation = 4;
            xw.IndentChar = ' ';
            
            xw.WriteStartDocument();
            xw.WriteStartElement("ExperimentConfiguration");
            // Anfang "SimulationInfo"
            xw.WriteStartElement("simulationInfo");
            
            xw.WriteStartElement("description");
            xw.WriteString(si.Description);
            //     xw.WriteString("Description");
            xw.WriteEndElement();
            
            xw.WriteStartElement("ownner");
            xw.WriteString(si.Owner);
            //     xw.WriteString("si.Ownner");
            xw.WriteEndElement();
            
            xw.WriteEndElement();//end simulationInfo
            // Ende "Simulationinfo"
            
            // Anfang "moduleinfos"
            xw.WriteStartElement("moduleinfos");
      
            foreach (ModuleInfo mi in mis){
                xw.WriteStartElement("moduleinfo");
                
                xw.WriteStartElement("moduleName");
                xw.WriteString(mi.ModuleName);
                xw.WriteEndElement();
                
                
                xw.WriteStartElement("assemblyName");
                xw.WriteString(mi.AssemblyName);
                xw.WriteEndElement();
                
                xw.WriteStartElement("className");
                xw.WriteString(mi.ClassName);
                xw.WriteEndElement();
                
                xw.WriteStartElement("parameters");
                
                foreach (Parameter pa in mi.Parameters){
                    xw.WriteStartElement("parameter");
                    
                    xw.WriteStartElement("type");
                    xw.WriteString(pa.Type);
                    xw.WriteEndElement();
                    
                    xw.WriteStartElement("name");
                    xw.WriteString(pa.Name);
                    xw.WriteEndElement();
                    
                    xw.WriteStartElement("value");
                    xw.WriteString(pa.Value);
                    xw.WriteEndElement();
                    
                    xw.WriteEndElement();
                    
                }
                xw.WriteEndElement();
                
          
                xw.WriteEndElement();
                
            }
      
            
            xw.WriteEndElement();
            
            // Ende "moduleinfos"
            

            xw.WriteEndElement();//end ExperimentConfiguration
            xw.WriteEndDocument();
            xw.Close();
      
            sw.Close();
            fs.Close();
        }
    }
    /// <summary>
    /// </summary> 
    /// @cond
    /// <author>Ting Wang</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: XMLWriter.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    class ModuleInfo
    {
        private string _moduleName;
        private string _assemblyName;
        private string _className;
        private Parameter[] _parameters;
        
        public ModuleInfo(string moduleName, string assmblyName, string className, Parameter[] parameters)
        {
            _moduleName = moduleName;
            _assemblyName = assmblyName;
            _className = className;
            _parameters = parameters;
        }
        public Parameter[] Parameters
        {
            get{
                return _parameters;
            }
            set{
                _parameters = value;
            }
        }
        
        public string ModuleName
        {
            get{
                return _moduleName;
            }
            set{
                _moduleName = value;
            }
        }
        public string AssemblyName
        {
            get{
                return _assemblyName;
            }
            set{
                _assemblyName = value;
            }
        }
        public string ClassName
        {
            get{
                return _className;
            }
            set{
                _className = value;
            }
        }
        
    }
    /// <summary>
    /// </summary> 
    /// @cond
    /// <author>Ting Wang</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: XMLWriter.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    class Parameter
    {
        private string _type;
        private string _name;
        private string _value;
        
        public Parameter(string type, string name, string value)
        {
        }
        public string Type
        {
            get{
                return _type;
            }
            set{
                _type = value;
            }
        }
        public string Name
        {
            get{
                return _name;
            }
            set{
                _name = value;
            }
        }
        public string Value
        {
            get{
                return _value;
            }
            set{
                _value = value;
            }
        }
        
    }
    
}
