/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.CoreApi.Individuals {

    [Serializable]
    public sealed class RucksackCharge : IComparable<RucksackCharge>, ICloneable<RucksackCharge>, ITransformable<RucksackCharge>, IFitnessReadable {

        public int fitness;

        public bool[] charge;

        internal RucksackCharge ()
        {
        }

        public double Fitness {
            get {
                return (double) -fitness;
            }
        }

        // ~IsWorseThan
        public int CompareTo (RucksackCharge other)
        {
            return other.fitness.CompareTo(this.fitness);
        }

        public void UseValuesOf (RucksackCharge other)
        {
            this.fitness = other.fitness;
            Array.Copy (other.charge, this.charge, this.charge.Length);
        }

        public RucksackCharge Clone ()
        {
            RucksackCharge clone = new RucksackCharge ();

            clone.charge = new bool[charge.Length];
            clone.fitness = fitness;

            Array.Copy (charge, clone.charge, charge.Length);

            return clone;
        }
    }
}
