/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

namespace EvoLab.CoreApi {
     
    public interface IModuleRegistry {
        void RegisterModule (string name, CycleModuleInfo registrationData);
        void UnregisterModule (string name);
        CycleModuleInfo GetRegistrationData (string name);
    }
    
    public class ModuleRegistry {
                
        private readonly Dictionary<string, CycleModuleInfo> registeredModules =
            new Dictionary<string,CycleModuleInfo>();
                
        public void RegisterModule (string name, CycleModuleInfo registrationData) {
            if (name == null)
                throw new ArgumentNullException ("name");
            if (registrationData == null)
                throw new ArgumentNullException("registrationData");
                        
            registeredModules.Add(name, registrationData);
        }
                
        public void UnregisterModule (string name) {
            if (name == null)
                throw new ArgumentNullException("name");
            if (!registeredModules.ContainsKey(name))
                throw new ArgumentException ("'"+name+"' is not registered");
                        
            registeredModules.Remove(name);
        }
                
        public CycleModuleInfo GetRegistrationData (string name) {
            if (!registeredModules.ContainsKey(name))
                throw new ArgumentException ("Module '"+name+"' hasn't registered itself.");
            return registeredModules[name];
        }
    }
        
}
