/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.ModuleTemplates {

    public abstract class Generator<I, P>
        : GeneratorBase<I, P>
        where P : PopFreePopulation<I>
        where I : class, ITransformable<I>
    {
        protected IModuleFitness<I, P> fitnessModule;
        
        public override void Init(
            string name, P population,
            IWorld world, IModuleLoader<I, P> moduleLoader)
        {
            base.Init(name, population, world, moduleLoader);
            
            fitnessModule = moduleLoader.GetDefaultEvaluator();
        }
        
        protected void Evaluate (I ind) {
            fitnessModule.Evaluate(ind);
        }
        
        protected abstract void GenerateNeighbour (I neighbour);

        public override void Run ()
        {
            population.Ind_B.UseValuesOf(population.Ind_A);
            GenerateNeighbour(population.Ind_B);
        }
    }
}
