/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.ModuleTemplates {

    public abstract class MutationBase<P>
    : IModuleMutation<P>
    {
        protected IWorld world;
        protected string name;
        protected P population;

        public virtual void Init(
            string name, P population,
            IWorld world)
        {
            if (name == null)
                throw new ArgumentNullException("name");
            if (population == null)
                throw new ArgumentNullException("population");
            if (world == null)
                throw new ArgumentNullException("world");
            
            this.name = name;
            this.world = world;
            this.population = population;
            
            world.RegisterModule(name, CycleModuleInfo.DefaultInfo);
        }

        public virtual void Dispose () {}

        public abstract void Run ();
    }
}
