/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.ModuleTemplates {

    public abstract class CSVOutput<I, P> : OutputBase<P>
        where P : IPopulation<I>
    {
        private string pathname;
                
        public override void Init(string name, P population, IWorld world, IOutputAccess output)
        {
            base.Init (name, population, world, output);
                        
            string filename = name + "_" + DateTime.Now.ToString("yyyy-MM-dd_HH-mm-ss") + "_" + world.UniqueId + ".csv";
            pathname = System.IO.Path.Combine(world.ResourceAccess.ResourceFolder, filename);

            //Creates the file making sure it doesn't exist yet.
            using (StreamWriter sw = new StreamWriter(File.Open(pathname, FileMode.CreateNew))) {
                WriteTableEntry(sw, "Generation");
                for (int i = 0; i < population.Pop.Length - 1; ++i) {
                    sw.Write(",");
                    WriteTableEntry(sw, "Ind"+i.ToString(System.Globalization.CultureInfo.InvariantCulture));
                }
                sw.Write(",");
                WriteTableEntry(sw, "Ind"+(population.Pop.Length - 1).ToString(System.Globalization.CultureInfo.InvariantCulture));
                sw.Write("\r\n");
            }
        }

        protected abstract string GetTableEntry (I Individual);
                
        public override void Run() {
            using (StreamWriter sw = new StreamWriter(File.Open(pathname, FileMode.Append))) {
                WriteTableEntry(sw, world.Generation.ToString(System.Globalization.CultureInfo.InvariantCulture));
                for (int i = 0; i < population.Pop.Length; i++) {
                    sw.Write(",");
                    WriteTableEntry(sw, GetTableEntry(population.Pop[i]));
                }
                sw.Write("\r\n");
            }
        }
                
        private void WriteTableEntry(StreamWriter sw, string entry) {
            if (entry.IndexOf('\n') != -1 || entry.IndexOf('\r') != -1
                || entry.IndexOf('\"') != -1 || entry.IndexOf(',') != -1)
                // quoting needed
                sw.Write("\"" + entry.Replace("\"", "\"\"") + "\"");
            else
                // no quoting needed
                sw.Write(entry);
        }
    }
}
