using System;

using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.Populations {
    
    public class PopFreeVectorPopulation<I> : PopFreePopulation<I>, VectorPopulation where I : class, ITransformable<I> {
        private readonly int vectorLength;        
        
        public PopFreeVectorPopulation (IndividualFactory<I> factory, ProblemType problemType, int vectorLength)
        :base (factory, problemType)
        {
            this.vectorLength = vectorLength;                    
        }
        
        public int VectorLength {
            get {
                return vectorLength;
            }
        }
    }
    
    public class PopFreePopulation<I> : IPopulation<I> where I : class, ITransformable<I> {

        private readonly I ind_A;
        private readonly I ind_B;
        
        private uint nothingBetterFoundCount = 0;
        private uint worseNotAcceptedCount = 0;
        
        private ProblemType problemType;
        private readonly IndividualFactory<I> factory;
        
        public PopFreePopulation (IndividualFactory<I> factory, ProblemType problemType) {
            if (factory == null)
                throw new ArgumentNullException ("factory");
            
            this.problemType = problemType;
            this.factory = factory;
            
            ind_A = factory.allocateNewIndividual();
            ind_B = factory.allocateNewIndividual();
            
            indContainer [0] = ind_A;
            indContainer [1] = ind_B;
        }
        
        public I AllocateNewIndividual () {
            return factory.allocateNewIndividual ();
        }
        
        public void BetterFound () {
            nothingBetterFoundCount = 0;
            worseNotAcceptedCount = 0;
        }
        
        public void NothingBetterFound () {
            nothingBetterFoundCount ++;
        }
        
        public void WorseAccepted () {
            worseNotAcceptedCount = 0;
        }
        
        public void WorseNotAccepted () {
            worseNotAcceptedCount ++;
        }
        
        public uint NothingBetterFoundCount {
            get {
                return nothingBetterFoundCount;
            }
        }
        
        public uint WorseNotAcceptedCount {
            get {
                return worseNotAcceptedCount;
            }
        }            
        
        public I Ind_A {
            get {
                return ind_A;            
            }
            set {
                this.ind_A.UseValuesOf(value);
            }
        }
       
        public I Ind_B {
            get {
                return ind_B;            
            }
            set {
                this.ind_B.UseValuesOf(value);
            }
        }
        
        public ProblemType ProblemType {
            get {
                return problemType;
            }
            set {
                this.problemType = value;
            }
        }
        
        private readonly I[] indContainer = new I[2];
        
        public I[] Pop {
            get {
                return indContainer;
            }
        }
       
    }
}
