/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;
using System.Collections.Generic;

namespace EvoLab.CoreApi.Problems {

    [Serializable]
    public struct RucksackObject {

        public readonly int use;
        public readonly int weight;

        public RucksackObject (int use, int weight)
        {
            this.use = use;
            this.weight = weight;
        }
    }

    [Serializable]
    public class RucksackProblem {

        public readonly int capacity;

        public readonly RucksackObject[] objects;
        
        public RucksackProblem (StreamReader sr)
        {
            string line = sr.ReadLine();

            this.capacity = Int32.Parse (line);

            List<RucksackObject> objectList = new List<RucksackObject> ();

            while (!sr.EndOfStream)
            {
                line = sr.ReadLine ();
                string[] rucksackobject = line.Split (new char[] {' '});
                objectList.Add (
                    new RucksackObject (
                        Int32.Parse (rucksackobject[0]), Int32.Parse (rucksackobject[1]) ));
            }

            this.objects = objectList.ToArray ();
        }

        public RucksackProblem ( int capacity, RucksackObject[] objects )
        {
            this.capacity = capacity;
            this.objects = objects;
        }

    }
}
