/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

/*
 * Methodenkommentare übernommen von Mersennetwister.cs
 */

using System;

namespace EvoLab.CoreApi.Random {

    /// <summary>
    /// Interface für Zufallszahlengeneratoren, die im EvoCore global
    /// verwendet werden, und somit beliebig ausgetauscht
    /// werden können.
    /// </summary> 
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2634 $</version>
    /// <id>$Id: IRandom.cs 2634 2008-04-28 15:28:13Z hofmandl $</id>
    /// @endcond
    public interface IRandom {

        /// <summary>
        /// Returns a random integer greater than or equal to zero and
        /// less than or equal to <c>MaxRandomInt</c>. 
        /// </summary>
        /// <returns>The next random integer.</returns>
        int Next();

        /// <summary>
        /// Returns a positive random integer less than the specified maximum.
        /// </summary>
        /// <param name="maxValue">The maximum value. Must be greater
        /// than or equal to zero.</param>
        /// <returns>A positive (including 0) random integer less
        /// than <c>maxValue</c>.</returns>
        int Next( int maxValue );

        /// <summary>
        /// Returns a random integer within the specified range.
        /// </summary>
        /// <param name="minValue">The lower bound.</param>
        /// <param name="maxValue">The upper bound.</param>
        /// <returns>A random integer greater than or equal to <c>minValue</c>, and less than
        /// <c>maxValue</c>. <c>minValue</c>, if the Values are equal.</returns>
        int Next( int minValue, int maxValue );

        /// <summary>
        /// Returns a random number between 0.0 and 1.0.
        /// </summary>
        /// <returns>A double-precision floating point number greater than or equal to 0.0, 
        /// and less than 1.0.</returns>
        double NextDouble();

        /// <summary>
        /// Returns "normally" distributed random value. N(0,1).
        /// </summary>
        /// <returns></returns>
        double NextGaussian();

    }

}
