/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.CoreApi
{
    /// <summary>
    /// Repr�sentiert ein Ereignis, das zwischen Simulationen
    /// kommuniziert wird.
    /// </summary>
    /// @cond
    /// <author>Tim Schuhmacher</author>
    /// <version>$Revision: 2634 $</version>
    /// <id>$Id: SimulationEvent.cs 2634 2008-04-28 15:28:13Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public struct SimulationEvent : IEquatable<SimulationEvent>
    {
        /// <summary>
        /// Die Art des Events.
        /// </summary>
        private SimEventType type;

        /// <summary>
        /// Der Absender des Events.
        /// </summary>
        private string senderId;

        /// <summary>
        /// Gibt den Absender zur�ck.
        /// </summary>
        public string SenderId
        {
            get {return this.senderId;}
        }

        /// <summary>
        /// Gibt die Art des Events zur�ck.
        /// </summary>
        public SimEventType Type
        {
            get { return this.type; }
        }

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="senderId">Der Absender.</param>
        /// <param name="type">Die Ereignis-Art.</param>
        public SimulationEvent(string senderId, SimEventType type)
        {
            if (senderId == null)
            {
                throw new ArgumentNullException("SenderId ist 'null'.");
            }
            this.senderId = senderId;
            this.type = type;
        }

        //TODO: See, whether Equals and GetHashCode-Overrides are necessary
        public override int GetHashCode ()
        {
            return type.GetHashCode ();
        }

        /// <summary>
        /// �berpr�fung auf Gleichheit.
        /// </summary>
        /// <param name="obj">Ein anderes SimulationEvent-Objekt.</param>
        /// <returns>True wenn der Inhalt der beiden Objekte gleich ist.Sonst False.</returns>
        public override bool Equals(object obj)
        {
            return this.Equals((SimulationEvent)obj);
        }

        /// <summary>
        /// �berpr�fung auf Gleichheit.
        /// </summary>
        /// <param name="simEvent">Ein anderes SimulationEvent-Objekt.</param>
        /// <returns>True wenn der Inhalt der beiden Objekte gleich ist.Sonst False.</returns>
        public bool Equals(SimulationEvent simEvent)
        {
            return (simEvent.senderId.Equals(this.senderId) &&
                simEvent.type.Equals(this.type));
        }
    }
}
