/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace EvoLab.Core.StandardModules.Universal
{
    /// <summary>
    /// Dies ist ein universell einsetzbares Cancelmodul, welches nach Erreichen einer bestimmten Generation abbricht.
    /// </summary> 
    [Author("Vasilios Grigas")]
    [Description("Universell einsetzbares Cancelmodul, das nach Erreichen einer bestimmten Generation abbricht.")]
    [Identifier("EA_CancelOnGenExceed")]
    [IntParameter("MAX_GEN", 1000)]
    public class CancelOnGenExceed<I,P> : CancelBase<P>
        where P : IPopulation<I>
        where I : class
    {
        private int myMaxGen;

        /// <summary>
        /// Bereitet dieses Modul f�r die Verwendung innerhalb einer Simulation vor.
        /// </summary>
        /// <param name="name">Der Name, mit dem sich dieses Modul identifizieren kann.</param>
        /// <param name="world">Die zu verwendende World f�r dieses Modul.</param>
        /// <exception cref="ArgumentOutOfRangeException">Tritt ein falls
        /// MAX_GEN kleiner 1 ist</exception>
        public override void Init(String name, P population, IWorld world, FinishDelegate finishDelegate)
        {
            base.Init(name, population, world, finishDelegate);
                        
            // Parameter MAXGEN wird eingelesen und kontrolliert:
            myMaxGen = world.GetLocalInteger(name, "MAX_GEN");
            if (myMaxGen < 1)
            {
                throw new ArgumentOutOfRangeException("Parameter 'MAX_GEN' < 1.");
            }
        }



        /// <summary>
        /// Prüft ob abgebrochen werden soll.
        /// </summary>
        public override void Run()
        {
            if (world.Generation > myMaxGen)
            {
                this.finish();
            }
        }
    }
}
