/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace FitnessFunctions {

    [Author("Lieferumfang von EvoLab")]
    [Description("Fitnessmodul für C-Funktionen")]
    [Identifier("Fitness_C")]

    public class Fitness_C<I, P> : Fitness<I, P> where I : IntVector where P : VectorPopulation, IPopulation<I>
    {
        private int vectorLength;
        private double[, ,] myDistances;

        public override void Init(string name, P population, IWorld world, ISimulationControl simControl)
        {
            base.Init (name, population, world, simControl);

            vectorLength = population.VectorLength;

            population.ProblemType = EvoLab.CoreApi.Populations.ProblemType.maximizeProblem;

            // Die Steigungsmatrix wird berechnet:
            myDistances = new double[vectorLength, vectorLength, vectorLength];

            for (int i = 0; i < vectorLength; i++)
            {
                for (int j = 0; j < vectorLength; j++)
                {
                    for (int k = 1; k < vectorLength; k++)
                    {
                        if (i != j)
                        {
                            myDistances[i, j, k] = (double)(System.Math.Abs(j - i)) / (double)k;
                        }
                    }
                }
            }
        }

        public override void Evaluate (I ind)
        {
            double aktFitness = 0.0;
            for (int i = 0; i < vectorLength; i++)
            {
                for (int j = 0; j < vectorLength; j++)
                {
                    // Damits schnell geht:
                    if (i > j)
                    {
                        aktFitness += myDistances[ind.vector[i], ind.vector[j], i - j];
                    }
                    else if (j > i)
                    {
                        aktFitness += myDistances[ind.vector[i], ind.vector[j], j - i];
                    }
                }
            }
            ind.Fitness = aktFitness;
        }
    }
}
