using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace FitnessFunctions {
    
    [Author("Lieferumfang von EvoLab")]
    [Description("Berechnet die Fitness für jedes Individuum")]
    [Identifier("Fitness_Rastrigin")]
    public sealed class Fitness_Rastrigin<I, P>
    : Fitness<I, P> where I : DoubleVector where P : IPopulation<I>
    {
        public override void Evaluate (I ind)
        {
            double sum = 0.0f;
            for (int i = 0; i < ind.vector.Length; ++i) {
                sum += RastriginSummand (ind.vector[i],i);
            }
            ind.Fitness = 10 * ind.vector.Length + sum;
        }

        private double RastriginSummand (double xi,int i)
        {
            xi = xi - (double)(i+1) * 1.5;
            return System.Math.Pow (xi, 2) - 10 * System.Math.Cos(2 * System.Math.PI * xi);
        }
    }
}
