using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace FitnessFunctions {
    
    [Author("Lieferumfang von EvoLab")]
    [Description("Berechnet die Fitness für jedes Individuum")]
    [Identifier("Fitness_Rosenbrock")]
    public sealed class Fitness_Rosenbrock<I, P>
    : Fitness<I, P> where I : DoubleVector where P : IPopulation<I>
    {
        public override void Evaluate(I ind)
        {
            double sum = 0.0f;
            for (int i = 0; i < ind.vector.Length - 1; ++i)
            {
                sum += RosenbrockSummand(ind.vector[i], ind.vector[i+1]);
            }
            ind.Fitness = sum;
        }

        private double RosenbrockSummand(double xi, double yi)
        {
            return 100.0d * (yi - xi * xi) * (yi - xi * xi) + (xi - 1) * (xi - 1);
        }
    }
}
