/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;

using System.Collections.Generic;
using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace FitnessFunctions
{
    [Author("Lieferumfang von EvoLab")]
    [Description("Fitnessmodul")]
    [Identifier("Fitness_TSP")]

    [GlobalFileParameter("TSP_FILE", "EvoLabTSP.tsp")]

    public class Fitness_TSP<I, P> : Fitness<I, P>
        where I : IntVector where P : IPopulation<I>
    {
        private EvoLab.CoreApi.Problems.TSP tsp;

        public override void Init(string name, P population, IWorld world, ISimulationControl simManagement)
        {
            base.Init(name, population, world, simManagement);

            if (!world.ContainsObject("tsp-file"))
            {
                using (System.IO.StreamReader sr =
                    world.ResourceAccess.OpenText(world.GetGlobalFileParameter("TSP_FILE")))
                {
                    tsp = new EvoLab.CoreApi.Problems.TSP(sr);
                }
                world.SetObject("tsp-file", tsp);
            }
            else
            {
                tsp = (EvoLab.CoreApi.Problems.TSP)world.GetObject("tsp-file");
            }

        }

        public override void Evaluate (I ind)
        {
            ind.Fitness = tsp.GetDistance (ind.vector);
        }
    }
}
