/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace PopFree
{
    [Identifier("PopFree_AcceptorHillClimbing")]
    
    public class PopFree_AcceptorHillClimbing<I, P> : Acceptor<I, P> where I : class, IComparable<I>, ITransformable<I> where P : PopFreeVectorPopulation<I> 
    {
        protected override I Choose(I old, I challenger)
        {
            // Konstante fr den Vergleich zweier Individuuen
            // A.CompareTo(B) liefert 1 => A > B, -1 => A < B
            int cmpTo = 1;

            // Konstante entsprechend des Problemtyps setzen
            if (population.ProblemType == ProblemType.maximizeProblem)
                cmpTo = -1;

            // Ist das Neue (challenger) besser wie das alte?
            if (old.CompareTo(challenger) == cmpTo) {
                // Ja es ist besser -> bernehmen und merken "Besseres gefunden"
                population.BetterFound();
                // Das zurckgegebene Individuum wird als neues gesetzt (hier das neue)
                return challenger;
            } else {
                // Nein -> merken "Kein besseres Gefunden" und "Es wurde das schlechte nicht akzeptiert"
                population.NothingBetterFound();
                population.WorseNotAccepted();
                // Das zurckgegebene Individuum wird als neues gesetzt (hier das alte)
                return old;
            }
        }
    }
}
