/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace EvoLab.Core.StandardModules.Universal
{

    [Identifier("PopFree_Permutation_GeneratorExchangeHillClimbing")]

    public class PopFree_Permutation_GeneratorExchangeHillClimbing<I, P> : Generator<I, P> where P : PopFreeVectorPopulation<I> where I : IntVector, ITransformable<I> 
    {
        // Konstante fr den Vergleich zweier Individuuen
        // A.CompareTo(B) liefert 1 => A > B, -1 => A < B
        int cmpTo = 1;
        
        public override void Init(String name, P population, IWorld world, IModuleLoader<I,P> moduleLoader)
        {
            base.Init(name, population, world, moduleLoader);


            // Konstante entsprechend des Problemtyps setzen
            if (population.ProblemType == ProblemType.maximizeProblem)
                cmpTo = -1;
        }
        
        protected override void GenerateNeighbour(I neighbour)
        {
            //Fitness des bergebenen Individuums berechnen.
            fitnessModule.Evaluate (neighbour);
            
            //Neues (leeres) Individuum erzeugen und mit dem Inhalt des bergebenen fllen
            I dummy = population.AllocateNewIndividual ();
            dummy.UseValuesOf (neighbour);
            
            //Alle mglichen Zweier-Tauschungen durchfhren 
            int dummyInt;
            for (int i=0; i<population.VectorLength - 1; ++i) {
                for (int j=i; j<population.VectorLength; ++j) {
                    dummyInt = dummy.vector[i];
                    dummy.vector[i] = dummy.vector[j];
                    dummy.vector[j] = dummyInt;
                    
                    fitnessModule.Evaluate (dummy);
                    
                    //Ist das neue Individuum besser - merken! 
                    if (neighbour.CompareTo(dummy) == cmpTo) {
                        neighbour.UseValuesOf(dummy);
                    }
                    
                    dummyInt = dummy.vector[i];
                    dummy.vector[i] = dummy.vector[j];
                    dummy.vector[j] = dummyInt;
                }
            }
        }
    }
}
