/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace EvoLab.Core.StandardModules.Universal
{

    [Identifier("PopFree_RealVector_GeneratorRealVectorAbsoluteDifference")]

    [DoubleParameter("Mittlere Abweichung pro Gen", 5.0d)]
    
    public class PopFree_RealVector_GeneratorAbsoluteDifference<I, P> : Generator<I, P> where P : PopFreeVectorPopulation<I> where I : DoubleVector, ITransformable<I> 
    {
        double deviation;        
        
        public override void Init(String name, P population, IWorld world, IModuleLoader<I,P> moduleLoader)
        {
            base.Init(name, population, world, moduleLoader);
            //Abweichung aus der Konfiguration auslesen
            deviation = world.GetLocalDouble(name, "Mittlere Abweichung pro Gen");
        }
        
        protected override void GenerateNeighbour(I neighbour)
        {
            for (int i=0; i<population.VectorLength; ++i) {
                //Angegebene Abweichung Normalverteilt auf jedes Element des Individuums addieren
                //Sehr simpler und auch nicht gerade guter Generator
                neighbour.vector[i] += world.RandomNumberGenerator.NextGaussian() * deviation;
            }
        }
    }
}
