/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace PopFree
{

    [Author("Lieferumfang von EvoLab")]
    [Description("Initialisierungsmodul")]
    [Identifier("PopFree_Initialization_Permutation")]

    [IntParameter("Laenge der Individuen", 25)]
    
    public sealed class PopFree_Initialization_Permutation : InitializationBase<IntVector, PopFreeVectorPopulation<IntVector>> 
    {
        public override PopFreeVectorPopulation<IntVector> CreatePopulation(string name, IWorld world, ISimulationControl simManagement)
        {
            if (world.GetLocalInteger(name, "Laenge der Individuen") < 0)
                throw new ArgumentOutOfRangeException (
                    "Laenge der Individuen darf nicht kleiner als 0 sein.");

            int length = world.GetLocalInteger(name, "Laenge der Individuen");

            PopFreeVectorPopulation<IntVector> pop = new PopFreeVectorPopulation<IntVector>(new IntVectorFactory(length), ProblemType.minimizeProblem, length);
            
            IntVector a = new IntVectorFactory(length).allocateNewIndividual();
            EvoLab.CoreApi.Operators.Initialization.RandomizeIntPermutation (world.RandomNumberGenerator, a.vector);
            
            pop.Ind_A = a;            
            
            return pop;
        }
    }

}
