using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Operators;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Populations;

namespace PopFree {

    [Author("Lieferumfang von EvoLab")]
    [Description("Initialisierungsmodul")]
    [Identifier("PopFree_Initialization_RealVector")]

    [DoubleParameter("Bereichs Minimum", -100.0)]
    [DoubleParameter("Bereichs Maximum", 100.0)]

    [IntParameter("Laenge der Individuen", 25)]

    public sealed class PopFree_Initialization_RealVector : InitializationBase<DoubleVector, PopFreeVectorPopulation<DoubleVector>> 
    {
        public override PopFreeVectorPopulation<DoubleVector> CreatePopulation(string name, IWorld world, ISimulationControl simManagement)
        {
            if (world.GetLocalInteger(name, "Laenge der Individuen") < 0)
                throw new ArgumentOutOfRangeException (
                    "Laenge der Individuen darf nicht kleiner als 0 sein.");

            int length = world.GetLocalInteger(name, "Laenge der Individuen");
            
            PopFreeVectorPopulation<DoubleVector> pop = new PopFreeVectorPopulation<DoubleVector>(new DoubleVectorFactory(length), ProblemType.minimizeProblem, length);
            
            DoubleVector a = new DoubleVectorFactory(length).allocateNewIndividual();
            
            Initialization.RandomizeDoubleArray(world.RandomNumberGenerator,
                                                    a.vector,
                                                    world.GetLocalDouble(name,"Bereichs Minimum"),
                                                    world.GetLocalDouble(name, "Bereichs Maximum")); 
            pop.Ind_A = a;
            
            return pop;
        }
    }
}
