using System;
using System.Collections.Generic;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace PopFree {

    [Identifier("PopFree_Simulation")]
    [OperatorParameter("OPERATOR_MODULES", new string[] { })]
    
    public class PopFree_Simulation<I, P> :  IModuleSimulation<I, P> where I : class, ITransformable<I> where P : PopFreePopulation<I> {
        protected IWorld world;
        protected P population;
        protected IModuleLoader<I, P> moduleLoader;
        protected string name;
        
        private readonly Dictionary<string, IModuleCore> loadedModules
            = new Dictionary<string, IModuleCore>();
        
        private readonly List<IModuleCore> cycle = new List<IModuleCore> ();        
        
        protected IModuleCore GetModule (string identifier) {
            IModuleCore module;
            if (!loadedModules.TryGetValue(identifier, out module)) {
                module = moduleLoader.LoadModule(identifier);
                loadedModules.Add(identifier, module);
            }
            return module;
        }
        
        public virtual void Init(
            string name,
            P population,
            IWorld world,
            IList<string> preCycleModules,
            IModuleLoader<I, P> moduleLoader)
        {
            if (name == null)
                throw new ArgumentNullException ("name");
            if (population == null)
                throw new ArgumentNullException ("population");
            if (world == null)
                throw new ArgumentNullException ("world");
            if (moduleLoader == null)
                throw new ArgumentNullException ("moduleLoader");
            if (preCycleModules == null)
                throw new ArgumentNullException ("preCycleModules");

            this.name = name;
            this.world = world;
            this.moduleLoader = moduleLoader;
            this.population = population;
            
            foreach (string module in preCycleModules) {
                GetModule(module).Run ();
            }
            
            string[] operator_modules = world.GetLocalOperatorParameter(name, "OPERATOR_MODULES");
            PrepareCycle(operator_modules);
        }
        
        private void PrepareCycle (string[] operators)
        {
            cycle.Clear();
            foreach (string moduleId in operators) {
                cycle.Add(GetModule(moduleId));
            }
        }
        
        public virtual void Run()
        {
            foreach (IModuleCore module in cycle)
            {
                module.Run();
            }
        }
                
        public void Dispose () {}
    }
    
}
