/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace PopFree
{
    [Author("Lieferumfang von EvoLab")]
    [Identifier("PopFree_AcceptanceTermination")]
    [IntParameter("MAX_NOTHING_BETTER_FOUND", 0)]    
    [IntParameter("MAX_WORSE_NOT_ACCEPTED", 0)]
    
    public class PopFree_AcceptanceTermination<I,P> : CancelBase<P>
        where P : PopFreePopulation<I>
        where I : class, ITransformable<I>
    {
        private int maxNothingBetterFound;
        private int maxWorseNotAccepted;
        
        public override void Init(String name, P population, IWorld world, FinishDelegate finishDelegate)
        {
            base.Init(name, population, world, finishDelegate);
                        
            maxNothingBetterFound = world.GetLocalInteger(name, "MAX_NOTHING_BETTER_FOUND");
            maxWorseNotAccepted = world.GetLocalInteger(name, "MAX_WORSE_NOT_ACCEPTED");
            if (maxNothingBetterFound < 0)
                throw new ArgumentOutOfRangeException("maxNothingBetterFound < 0");
            if (maxWorseNotAccepted < 0)
                throw new ArgumentOutOfRangeException("maxWorseNotAccepted < 0");
        }

        public override void Run()
        {
            if ((population.NothingBetterFoundCount > maxNothingBetterFound && maxNothingBetterFound != 0)
                || (population.WorseNotAcceptedCount > maxWorseNotAccepted && maxWorseNotAccepted != 0))
                this.finish();
        }
    }


}
