/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;

using System.Collections.Generic;
using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace TSP
{
    /// <summary>
    /// Dies ist ein Outputmodul f�r die Permuations-Realisierung "EvoLabPermutation".
    /// Das Modul f�gt immer dem selben Datensatz (Nr 1) das aktuell beste Individuum hinzu,
    /// falls dieses besser ist als das bisher beste. Zudem wird vor jeden neue Eintrag die
    /// aktuelle Generation ausgegeben.
    /// </summary> 

    [Author("Lieferumfang von EvoLab")]
    [Description("Fügt neues Bestes einer Liste hinzu.")]
    [Identifier("EA_TSP_OutputAddTheBestOne")]

    [IntParameter("OUTPUT_INTERVAL", 1)]
        
    public class OutputAddTheBestOne<I, P> : OutputBase<P>
        where I : IntVector where P : IPopulation<I> 
    {
        private int outputInterval;

        public override void Init(string name, P population, IWorld world, IOutputAccess fileAccess)
        {
            base.Init (name, population, world, fileAccess);
                        
            outputInterval = world.GetLocalInteger(name, "OUTPUT_INTERVAL");
        }

                
        public override void Run()
        {
            if (((world.Generation - 1) % outputInterval) == 0) {
                IntVector newBest = population.Pop[0];

                foreach (I ind in population.Pop) {
                    if (newBest.CompareTo(ind) == 1) {
                        newBest = ind;
                    }
                }

                string output = world.Generation.ToString();
                foreach (int i in newBest.vector)
                {
                    output += ";" + i.ToString();
                }
                output += "|";              
                                        
                fileAccess.WriteDataSet(output, 17, name);
            }
        }
        /*
          private void DumpPermutations (IntVector [] inds)
          {
          for (int i=0; i<inds.Length; ++i) {
          if (inds[i] != null) {
          Console.WriteLine ("----------------------------------------");
          Console.WriteLine ("Fitness: " + inds[i].Fitness);
          for (int j=0; j<population.VectorLength; ++j) {
          int val = inds[i].vector[j];
          string valstring;
          if (val < 10)
          valstring = val + " ";
          else
          valstring = val.ToString ();
          Console.Write (valstring + " ");
          }
          Console.WriteLine ();
          }
          }
          }
        */
    }
}
