/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Programs)                                   *
 *                                                                            *
 *  EvoLab is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation, either version 3 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with EvoLab.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace TSP
{
    [Author("Lieferumfang von EvoLab")]
    [Description("Initialisierungsmodul")]
    [Identifier("EA_TSP_Initialization")]

    [GlobalFileParameter("TSP_FILE", "EvoLabTSP.tsp")]

    [IntParameter("μ", 100)]
    [IntParameter("λ", 500)]

    public sealed class Initialization : InitializationBase<IntVector, PermutationPopulation> 
    {
        public override PermutationPopulation CreatePopulation(string name, IWorld world, ISimulationControl simManagement)
        {
            if (world.GetLocalInteger(name, "μ") < 1)
                throw new ArgumentOutOfRangeException (
                    "μ ist kleiner als 1. Keine leere Population möglich.");

            if (world.GetLocalInteger(name, "λ") < 1)
                throw new ArgumentOutOfRangeException (
                    "λ ist kleiner als 0. Keine negativ-anzahlige Nachkommenschaft möglich.");

            EvoLab.CoreApi.Problems.TSP tsp = null;

            using (System.IO.StreamReader sr = 
                world.ResourceAccess.OpenText(world.GetGlobalFileParameter("TSP_FILE")))
            {
                tsp = new EvoLab.CoreApi.Problems.TSP(sr);
            }

            int μ = world.GetLocalInteger(name, "μ");
            PermutationPopulation population = new PermutationPopulation(
                tsp.length, μ, world.GetLocalInteger(name, "λ"));

            world.SetObject("tsp-file", tsp);

            foreach (IntVector ind in population.Pop) {
                EvoLab.CoreApi.Operators.Initialization.RandomizeIntPermutation (world.RandomNumberGenerator, ind.vector);
            }
                        
            return population;
        }
    }
}
